==> speechanalytic-billing_0.1.4_database.tar.gz: database/mssql/procedures/GetAccountSubAccountVCCBalance.sql <==
SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAccountSubAccountVCCBalance] (
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL,
	@SystemService VARCHAR(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	-- VERIFICAMOS SI EXISTE UNA PLATAFORMA CON EL PLATFORMHASH INGRESADO
	IF NOT EXISTS(
		SELECT P.[PlatformHash] 
			FROM dbo.[Platform] P 
			WHERE P.[PlatformHash] = @PlatformHash
				AND P.[InstallationId] = @InstallationId
	)
	BEGIN
		-- SI NO EXISTE LA PLATAFORMA ENVIAMOS UN MENSAJE DE ERROR
		RAISERROR('DATABABASE_ERROR_NOT_FOUND_PLATAFORM', 16, 16)
		RETURN
	END

	-- VERIFICAMOS SI SUBACCOUNID ES NULL PARA LA VALIDACION DE CUENTA/SUBCUENTA
	IF @SubAccountId IS NULL 
	BEGIN
		-- SI SUBCUENTA ES NULL, SOLO VERIFICAMOS SI EXISTE LA CUENTA
		IF NOT EXISTS(
			SELECT A.[AccountId] 
				FROM dbo.[Account] A 
				WHERE A.[PlatformHash] = @PlatformHash
					AND A.[InstallationId] = @InstallationId 
					AND A.[AccountId] = @AccountId
					AND A.[SubAccountId] IS NULL
		)
		BEGIN
			-- SI NO EXISTE LA CUENTA ENVIAMOS UN MENSAJE DE ERROR
			RAISERROR('DATABABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
			RETURN
		END
		ELSE 
		BEGIN
			-- VERIFICAMOS SI EXISTE EL VCC ASOCIADO A LA CUENTA
			IF NOT EXISTS(
				SELECT AV.[VCC] 
					FROM dbo.[AccountVCC] AV 
					WHERE AV.[InstallationId] = @InstallationId 
						AND AV.[AccountId] = @AccountId
						AND AV.[SubAccountId] IS NULL
						AND AV.[VCC] = @VCC
			)	
			BEGIN
				-- SI NO EXISTE LA CUENTA ENVIAMOS UN MENSAJE DE ERROR
				RAISERROR('DATABABASE_ERROR_NOT_FOUND_ACCOUNT_VCC', 16, 16)
				RETURN
			END
		END
	END
	ELSE
	BEGIN 
		-- SI SUBCUENTA ES DIFERENTE DE NULL, VERIFICAMOS SI EXISTE LA CUENTA CON SUBCUENTA ASOCIADA
		IF NOT EXISTS(
			SELECT A.[SubAccountId] 
				FROM dbo.[Account] A 
				WHERE A.[PlatformHash] = @PlatformHash
					AND A.[InstallationId] = @InstallationId 
					AND A.[AccountId] = @AccountId
					AND A.[SubAccountId] = @SubAccountId
		)
		BEGIN
			-- SI NO EXISTE LA PLATAFORMA ENVIAMOS UN MENSAJE DE ERROR
			RAISERROR('DATABABASE_ERROR_NOT_FOUND_SUBACCOUNT', 16, 16)
			RETURN
		END
		ELSE 
		BEGIN
			-- VERIFICAMOS SI EXISTE EL VCC ASOCIADO A LA CUENTA-SUBCUENTA
			IF NOT EXISTS(
				SELECT AV.[VCC] 
					FROM dbo.[AccountVCC] AV 
					WHERE AV.[InstallationId] = @InstallationId 
						AND AV.[AccountId] = @AccountId
						AND AV.[SubAccountId] = @SubAccountId
						AND AV.[VCC] = @VCC
			)	
			BEGIN
				-- SI NO EXISTE LA CUENTA ENVIAMOS UN MENSAJE DE ERROR
				RAISERROR('DATABABASE_ERROR_NOT_FOUND_SUBACCOUNT_VCC', 16, 16)
				RETURN
			END
		END
	END;

	-- DECLARAMOS LAS CTE QUE JUNTARAN LA DATA REQUERIDA PARA EL RESULT
	WITH CTE_BATCHES_DETAILED_RATED AS (
		-- HACEMOS SELECT A LOS BATCHES CON POR BATACHID, SYSTEMSERVICE, CON SALDO RETENIDO Y SALDO CONSUMIDO
		SELECT ASAVST.[InstallationId],
				ASAVBM.[AccountId],
				ASAVBM.[SubAccountId],
				ASAVST.[VCC],
				ASAVST.[BatchID],
				ASAVST.[SystemService],
				ASAVBM.[Amount] AS 'Estimated',
				SUM(ASAVDT.[Rate]) AS 'Rated',
				ASAVST.[CreatedDate],
				COUNT(*) AS 'Details'
			FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK)
			LEFT JOIN [dbo].[AccountSubAccountVCCDetailedTransaction] ASAVDT (NOLOCK) 
				ON ASAVDT.[BatchId] = ASAVST.[BatchID]
					AND ASAVDT.[SystemService] = ASAVST.[SystemService]
			LEFT JOIN [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK)
				ON ASAVBM.[VCC] = ASAVST.[VCC]
					AND ASAVBM.[BatchId] = ASAVST.[BatchID]
					AND ASAVBM.[Wallet] = ASAVST.[SystemService]
			WHERE ASAVST.[InstallationId] = @InstallationId
				AND ASAVBM.[AccountId] = @AccountId
				AND (
					(@SubAccountId IS NULL AND ASAVBM.[SubAccountId] IS NULL)
					OR (ASAVBM.[SubAccountId] = @SubAccountId)
				)
				AND ASAVBM.[VCC] = @VCC
				AND ASAVBM.[Type] = 'RETAIN'
				--AND (ASAVST.[Status] <> 'FINISHED' AND ASAVST.[Status] NOT LIKE 'CANCELLED')
				AND (ASAVST.[Status] NOT LIKE 'FINISHED%')
				AND GETUTCDATE() < DATEADD(DAY, 5, ASAVST.[CreatedDate])
				AND ASAVDT.[IsDemo] = 0
			GROUP BY ASAVST.[InstallationId],
						ASAVBM.[AccountId],
						ASAVBM.[SubAccountId],
						ASAVST.[VCC],
						ASAVST.[BatchID],
						ASAVST.[SystemService],
						ASAVBM.[Amount],
						ASAVST.[CreatedDate]
	),
	CTE_BATCHES_SUMMARY_RATED AS (
		-- HACEMOS SELECT AL RESULTADO ANTERIOR PARA SUMARIZAR EL RATED, ESTIMATED Y DIFFERENCE
		SELECT CTEBDR.[InstallationId],
				CTEBDR.[AccountId],
				CTEBDR.[SubAccountId],
				CTEBDR.[VCC],
				CTEBDR.[SystemService],
				SUM(CTEBDR.[Estimated]) AS 'Estimated',
				SUM(CTEBDR.[Rated]) AS 'Rated'
			FROM CTE_BATCHES_DETAILED_RATED CTEBDR 
			GROUP BY CTEBDR.[InstallationId],
						CTEBDR.[AccountId],
						CTEBDR.[SubAccountId],
						CTEBDR.[VCC],
						CTEBDR.[SystemService]
	)
	
	-- HACEMOS LA BUSQUEDA DE LOS SALDOS DEL CUENTA-SUBCUENTA-VCC
	SELECT AVB.[AccountId]														'accountId', 
			AVB.[SubAccountId]													'subAccountId',
			AVB.[SystemService]													'systemService',
			AVB.[CountableBalance]												'countableBalance',
			AVB.[AvailableBalance]												'availableBalance',
			AVB.[RetainedBalance]												'retainedBalance',
			-- ISNULL(CTEBMR.[Estimated], 0)										'estimated',
			-- ISNULL(CTEBMR.[Rated], 0)											'rated',
			ISNULL(ABS(CTEBMR.[Estimated] - CTEBMR.[Rated]), 0)					'difference',
			CASE WHEN CTEBMR.[Rated] > CTEBMR.[Estimated] THEN 1 ELSE 0 END		'exceeded'
		FROM dbo.[AccountVCCBalance] AVB (NOLOCK)
		LEFT JOIN CTE_BATCHES_SUMMARY_RATED CTEBMR 
			ON CTEBMR.[InstallationId] = AVB.[InstallationId]
				AND CTEBMR.[AccountId] = AVB.[AccountId]
				AND (
					(CTEBMR.[SubAccountId] IS NULL AND AVB.[SubAccountId] IS NULL)
					OR (CTEBMR.[SubAccountId] = AVB.[SubAccountId])
				)
				AND CTEBMR.[VCC] = AVB.[VCC]
				AND CTEBMR.[SystemService] = AVB.[SystemService]
		WHERE AVB.[InstallationId] = @InstallationId 
			AND AVB.[AccountId] = @AccountId
			AND (
				(@SubAccountId IS NULL AND AVB.[SubAccountId] IS NULL)
				OR (AVB.[SubAccountId] = @SubAccountId)
			) 
			AND AVB.[VCC] = @VCC
			AND AVB.[SystemService] = ISNULL(@SystemService, AVB.[SystemService]) 

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

==> speechanalytic-billing_1.0.1_database.tar.gz: database/mssql/procedures/GetAccountSubAccountVCCBalance.sql <==
SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAccountSubAccountVCCBalance] (
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL,
	@SystemService VARCHAR(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	-- VERIFICAMOS SI EXISTE UNA PLATAFORMA CON EL PLATFORMHASH INGRESADO
	IF NOT EXISTS(
		SELECT P.[PlatformHash] 
			FROM dbo.[Platform] P 
			WHERE P.[PlatformHash] = @PlatformHash
				AND P.[InstallationId] = @InstallationId
	)
	BEGIN
		-- SI NO EXISTE LA PLATAFORMA ENVIAMOS UN MENSAJE DE ERROR
		RAISERROR('DATABASE_ERROR_NOT_FOUND_PLATAFORM', 16, 16)
		RETURN
	END

	-- VERIFICAMOS SI SUBACCOUNID ES NULL PARA LA VALIDACION DE CUENTA/SUBCUENTA
	IF @SubAccountId IS NULL 
	BEGIN
		-- SI SUBCUENTA ES NULL, SOLO VERIFICAMOS SI EXISTE LA CUENTA
		IF NOT EXISTS(
			SELECT A.[AccountId] 
				FROM dbo.[Account] A 
				WHERE A.[PlatformHash] = @PlatformHash
					AND A.[InstallationId] = @InstallationId 
					AND A.[AccountId] = @AccountId
					AND A.[SubAccountId] IS NULL
		)
		BEGIN
			-- SI NO EXISTE LA CUENTA ENVIAMOS UN MENSAJE DE ERROR
			RAISERROR('DATABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
			RETURN
		END
		ELSE 
		BEGIN
			-- VERIFICAMOS SI EXISTE EL VCC ASOCIADO A LA CUENTA
			IF NOT EXISTS(
				SELECT AV.[VCC] 
					FROM dbo.[AccountVCC] AV 
					WHERE AV.[InstallationId] = @InstallationId 
						AND AV.[AccountId] = @AccountId
						AND AV.[SubAccountId] IS NULL
						AND AV.[VCC] = @VCC
			)	
			BEGIN
				-- SI NO EXISTE LA CUENTA ENVIAMOS UN MENSAJE DE ERROR
				RAISERROR('DATABASE_ERROR_NOT_FOUND_ACCOUNT_VCC', 16, 16)
				RETURN
			END
		END
	END
	ELSE
	BEGIN 
		-- SI SUBCUENTA ES DIFERENTE DE NULL, VERIFICAMOS SI EXISTE LA CUENTA CON SUBCUENTA ASOCIADA
		IF NOT EXISTS(
			SELECT A.[SubAccountId] 
				FROM dbo.[Account] A 
				WHERE A.[PlatformHash] = @PlatformHash
					AND A.[InstallationId] = @InstallationId 
					AND A.[AccountId] = @AccountId
					AND A.[SubAccountId] = @SubAccountId
		)
		BEGIN
			-- SI NO EXISTE LA PLATAFORMA ENVIAMOS UN MENSAJE DE ERROR
			RAISERROR('DATABASE_ERROR_NOT_FOUND_SUBACCOUNT', 16, 16)
			RETURN
		END
		ELSE 
		BEGIN
			-- VERIFICAMOS SI EXISTE EL VCC ASOCIADO A LA CUENTA-SUBCUENTA
			IF NOT EXISTS(
				SELECT AV.[VCC] 
					FROM dbo.[AccountVCC] AV 
					WHERE AV.[InstallationId] = @InstallationId 
						AND AV.[AccountId] = @AccountId
						AND AV.[SubAccountId] = @SubAccountId
						AND AV.[VCC] = @VCC
			)	
			BEGIN
				-- SI NO EXISTE LA CUENTA ENVIAMOS UN MENSAJE DE ERROR
				RAISERROR('DATABASE_ERROR_NOT_FOUND_SUBACCOUNT_VCC', 16, 16)
				RETURN
			END
		END
	END;

	-- DECLARAMOS LAS CTE QUE JUNTARAN LA DATA REQUERIDA PARA EL RESULT
	WITH CTE_BATCHES_DETAILED_RATED AS (
		-- HACEMOS SELECT A LOS BATCHES CON POR BATACHID, SYSTEMSERVICE, CON SALDO RETENIDO Y SALDO CONSUMIDO
		SELECT ASAVST.[InstallationId],
				ASAVBM.[AccountId],
				ASAVBM.[SubAccountId],
				ASAVST.[VCC],
				ASAVST.[BatchID],
				ASAVST.[SystemService],
				ASAVBM.[Amount] AS 'Estimated',
				SUM(ASAVDT.[Rate]) AS 'Rated',
				ASAVST.[CreatedDate],
				COUNT(*) AS 'Details'
			FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK)
			LEFT JOIN [dbo].[AccountSubAccountVCCDetailedTransaction] ASAVDT (NOLOCK) 
				ON ASAVDT.[BatchId] = ASAVST.[BatchID]
					AND ASAVDT.[SystemService] = ASAVST.[SystemService]
			LEFT JOIN [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK)
				ON ASAVBM.[VCC] = ASAVST.[VCC]
					AND ASAVBM.[BatchId] = ASAVST.[BatchID]
					AND ASAVBM.[Wallet] = ASAVST.[SystemService]
			WHERE ASAVST.[InstallationId] = @InstallationId
				AND ASAVBM.[AccountId] = @AccountId
				AND (
					(@SubAccountId IS NULL AND ASAVBM.[SubAccountId] IS NULL)
					OR (ASAVBM.[SubAccountId] = @SubAccountId)
				)
				AND ASAVBM.[VCC] = @VCC
				AND ASAVBM.[Type] = 'RETAIN'
				--AND (ASAVST.[Status] <> 'FINISHED' AND ASAVST.[Status] NOT LIKE 'CANCELLED')
				AND (ASAVST.[Status] NOT LIKE 'FINISHED%')
				AND GETUTCDATE() < DATEADD(DAY, 5, ASAVST.[CreatedDate])
				AND ASAVDT.[IsDemo] = 0
			GROUP BY ASAVST.[InstallationId],
						ASAVBM.[AccountId],
						ASAVBM.[SubAccountId],
						ASAVST.[VCC],
						ASAVST.[BatchID],
						ASAVST.[SystemService],
						ASAVBM.[Amount],
						ASAVST.[CreatedDate]
	),
	CTE_BATCHES_SUMMARY_RATED AS (
		-- HACEMOS SELECT AL RESULTADO ANTERIOR PARA SUMARIZAR EL RATED, ESTIMATED Y DIFFERENCE
		SELECT CTEBDR.[InstallationId],
				CTEBDR.[AccountId],
				CTEBDR.[SubAccountId],
				CTEBDR.[VCC],
				CTEBDR.[SystemService],
				SUM(CTEBDR.[Estimated]) AS 'Estimated',
				SUM(CTEBDR.[Rated]) AS 'Rated'
			FROM CTE_BATCHES_DETAILED_RATED CTEBDR 
			GROUP BY CTEBDR.[InstallationId],
						CTEBDR.[AccountId],
						CTEBDR.[SubAccountId],
						CTEBDR.[VCC],
						CTEBDR.[SystemService]
	)
	
	-- HACEMOS LA BUSQUEDA DE LOS SALDOS DEL CUENTA-SUBCUENTA-VCC
	SELECT AVB.[AccountId]														'accountId', 
			AVB.[SubAccountId]													'subAccountId',
			C.[Code]															'currencyCode',
			C.[Symbol]															'currencySymbol',
			AVB.[SystemService]													'systemService',
			AVB.[CountableBalance]												'countableBalance',
			AVB.[AvailableBalance]												'availableBalance',
			AVB.[RetainedBalance]												'retainedBalance',
			-- ISNULL(CTEBMR.[Estimated], 0)										'estimated',
			-- ISNULL(CTEBMR.[Rated], 0)											'rated',
			ISNULL(ABS(CTEBMR.[Estimated] - CTEBMR.[Rated]), 0)					'difference',
			CASE WHEN CTEBMR.[Rated] > CTEBMR.[Estimated] THEN 1 ELSE 0 END		'exceeded'
		FROM dbo.[AccountVCCBalance] AVB (NOLOCK)
		LEFT JOIN CTE_BATCHES_SUMMARY_RATED CTEBMR 
			ON CTEBMR.[InstallationId] = AVB.[InstallationId]
				AND CTEBMR.[AccountId] = AVB.[AccountId]
				AND (
					(CTEBMR.[SubAccountId] IS NULL AND AVB.[SubAccountId] IS NULL)
					OR (CTEBMR.[SubAccountId] = AVB.[SubAccountId])
				)
				AND CTEBMR.[VCC] = AVB.[VCC]
				AND CTEBMR.[SystemService] = AVB.[SystemService]
		INNER JOIN [dbo].[Account] A (NOLOCK)
			ON A.[InstallationId] = AVB.[InstallationId]
				AND A.[AccountId] = AVB.[AccountId]
				AND (
					A.[SubAccountId] = AVB.[SubAccountId]
					OR 
					(
						A.[SubAccountId] IS NULL
						AND 
						AVB.[SubAccountId] IS NULL
					)
				)
		INNER JOIN [dbo].[Currency] C (NOLOCK)
			ON C.[Code] = A.[CurrencyCode]
		WHERE AVB.[InstallationId] = @InstallationId 
			AND AVB.[AccountId] = @AccountId
			AND (
				(@SubAccountId IS NULL AND AVB.[SubAccountId] IS NULL)
				OR (AVB.[SubAccountId] = @SubAccountId)
			) 
			AND AVB.[VCC] = @VCC
			AND AVB.[SystemService] = ISNULL(@SystemService, AVB.[SystemService]) 

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
